import pytest
from unittest.mock import MagicMock
from relevanceai.resources.agents import AgentsManager
from relevanceai.resources.agent import Agent


class TestAgentsManager:
    @pytest.fixture
    def mock_client(self):
        """Fixture to create a mock RelevanceAI client."""
        return MagicMock()

    @pytest.fixture
    def agents_manager(self, mock_client):
        """Fixture to create an AgentsManager instance."""
        return AgentsManager(client=mock_client)

    def test_list_agents(self, agents_manager):
        """Test listing agents with successful response."""
        mock_response = MagicMock()
        mock_response.json.return_value = {
            "results": [
                {"agent_id": "agent-1", "name": "Agent 1", "_id": "id-1", "project": "default"},
                {"agent_id": "agent-2", "name": None, "_id": "id-2", "project": "default"},
                {"agent_id": "agent-3", "name": "Agent 3", "_id": "id-3", "project": "default"}
            ]
        }
        agents_manager._post = MagicMock(return_value=mock_response)

        result = agents_manager.list_agents()

        agents_manager._post.assert_called_once_with("agents/list")
        assert len(result) == 3
        assert all(isinstance(agent, Agent) for agent in result)
        assert [agent.metadata.name for agent in result] == ["Agent 1", "Agent 3", None]

    def test_retrieve_agent(self, agents_manager):
        """Test retrieving a specific agent."""
        mock_response = MagicMock()
        mock_response.json.return_value = {
            "agent": {
                "agent_id": "test-agent",
                "name": "Test Agent",
                "_id": "test-id",
                "project": "default"
            }
        }
        agents_manager._get = MagicMock(return_value=mock_response)

        result = agents_manager.retrieve_agent("test-agent")

        agents_manager._get.assert_called_once_with("agents/test-agent/get")
        assert isinstance(result, Agent)
        assert result.agent_id == "test-agent"
        assert result.metadata.name == "Test Agent"

    def test_upsert_agent(self, agents_manager):
        """Test upserting an agent with various parameters."""
        mock_upsert_response = MagicMock()
        mock_upsert_response.json.return_value = {"agent_id": "new-agent"}
        agents_manager._post = MagicMock(return_value=mock_upsert_response)

        mock_agent = Agent(
            client=agents_manager._client,
            agent_id="new-agent",
            name="New Agent",
            system_prompt="Test prompt",
            model="gpt-4",
            temperature=0.7,
            _id="new-id",
            project="default"
        )
        agents_manager.retrieve_agent = MagicMock(return_value=mock_agent)

        result = agents_manager.upsert_agent(
            agent_id="new-agent",
            name="New Agent",
            system_prompt="Test prompt",
            model="gpt-4",
            temperature=0.7
        )

        agents_manager._post.assert_called_once_with(
            "/agents/upsert",
            body={
                "agent_id": "new-agent",
                "name": "New Agent",
                "system_prompt": "Test prompt",
                "model": "gpt-4",
                "temperature": 0.7
            }
        )
        agents_manager.retrieve_agent.assert_called_once_with("new-agent")
        assert isinstance(result, Agent)
        assert result.agent_id == "new-agent"

    def test_upsert_agent_minimal(self, agents_manager):
        """Test upserting an agent with minimal parameters."""
        mock_upsert_response = MagicMock()
        mock_upsert_response.json.return_value = {"agent_id": "new-agent"}
        agents_manager._post = MagicMock(return_value=mock_upsert_response)

        mock_agent = Agent(
            client=agents_manager._client,
            agent_id="new-agent",
            _id="new-id",
            project="default"
        )
        agents_manager.retrieve_agent = MagicMock(return_value=mock_agent)

        result = agents_manager.upsert_agent(agent_id="new-agent")

        agents_manager._post.assert_called_once_with(
            "/agents/upsert",
            body={"agent_id": "new-agent"}
        )
        assert isinstance(result, Agent)
        assert result.agent_id == "new-agent"

    def test_delete_agent(self, agents_manager):
        """Test deleting an agent."""
        mock_response = MagicMock()
        mock_response.status_code = 200
        agents_manager._post = MagicMock(return_value=mock_response)

        result = agents_manager.delete_agent("test-agent")

        agents_manager._post.assert_called_once_with("agents/test-agent/delete")
        assert result is True

    def test_delete_agent_failure(self, agents_manager):
        """Test deleting an agent with non-200 response."""
        mock_response = MagicMock()
        mock_response.status_code = 404
        agents_manager._post = MagicMock(return_value=mock_response)

        result = agents_manager.delete_agent("test-agent")

        agents_manager._post.assert_called_once_with("agents/test-agent/delete")
        assert result is False



from .agents import AgentsManager, AsyncAgentsManager
from .tasks import Tasks, AsyncTasks
from .tools import ToolsManager, AsyncToolsManager
from .knowledge import Knowledge, AsyncKnowledge
from .oauth import OAuthManager



"""
Trigger and poll for task outputs of agents
"""

from textwrap import dedent
import time 
from dotenv import load_dotenv
load_dotenv()

from relevanceai import RelevanceAI

client = RelevanceAI()

agent_id = "xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
my_agent = client.agents.retrieve_agent(agent_id=agent_id)

message = dedent(f"""
Research the following company:        
RelevanceAI relevanceai.com      
""")

task = my_agent.trigger_task(
    message=message
)

while not my_agent.get_task_output_preview(task.conversation_id): 
    print("polling...\n")
    time.sleep(5)

task_output_preview = my_agent.get_task_output_preview(agent_id, task.conversation_id)

print(task_output_preview["answer"])


import pytest
from relevanceai import RelevanceAI

class TestRelevanceAI:
    def test_client_init(self):
        client = RelevanceAI(api_key="test_key", region="test_region", project="test_project")
        assert client.api_key == "test_key"
        assert client.region == "test_region"
        assert client.project == "test_project"



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field, constr


class DefaultOutputValue(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    original_key: str
    updated_key: Optional[str] = None
    value: Any


class Step(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    name: str
    transformation: str
    params: Dict[str, Any]
    saved_params: Optional[Dict[str, Any]] = None
    output: Optional[Dict[str, Any]] = None
    default_output_values: Optional[List[DefaultOutputValue]] = None
    continue_on_error: Optional[bool] = None
    use_fallback_on_skip: Optional[bool] = None
    foreach: Optional[Union[str, List]] = None
    if_: Optional[Union[str, bool]] = Field(None, alias='if')
    display_name: Optional[str] = None


class Transformations(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    steps: List[Step]
    output: Optional[Dict[str, str]] = None


class HostType(Enum):
    batch = 'batch'
    lambda_ = 'lambda'
    instant_workflows = 'instant-workflows'
    none = 'none'


class Properties(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    params: Dict[str, Any]
    workflow_id: str
    host_type: Optional[HostType] = 'batch'
    dataset_id: Optional[str] = None
    version: Optional[str] = None


class Template(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    transformations: Dict[str, Transformations]


class Type(Enum):
    transformation = 'transformation'


class Categories(Enum):
    boolean_True = True


class Tags(BaseModel):
    type: Optional[Type] = None
    categories: Optional[Dict[str, Categories]] = None
    integration_source: Optional[str] = Field(
        None,
        description="The source of the integration. For example, 'Knowledge: Linear', which imports data from Linear, this would be 'linear'.",
    )


class Metadata(BaseModel):
    field_order: Optional[List[str]] = None


class ContentType(Enum):
    json = 'json'
    json_list = 'json_list'
    long_text = 'long_text'
    short_text = 'short_text'
    file_url = 'file_url'
    file_urls = 'file_urls'
    llm_prompt = 'llm_prompt'
    speech = 'speech'
    code = 'code'
    dataset_id = 'dataset_id'
    knowledge_set = 'knowledge_set'
    markdown = 'markdown'
    chain_id = 'chain_id'
    chain_params = 'chain_params'
    file_to_text = 'file_to_text'
    file_to_text_llm_friendly = 'file_to_text_llm_friendly'
    memory_optimizer = 'memory_optimizer'
    memory = 'memory'
    table = 'table'
    agent_id = 'agent_id'
    api_key = 'api_key'
    key_value_input = 'key_value_input'
    knowledge_editor = 'knowledge_editor'
    oauth_account = 'oauth_account'
    datetime = 'datetime'
    api_selector = 'api_selector'
    llm_model_selector = 'llm_model_selector'
    colour_picker = 'colour_picker'
    conditional = 'conditional'
    external_field = 'external_field'
    tool_approval = 'tool_approval'


class ApiSelectorType(Enum):
    finetuning_model_select = 'finetuning_model_select'
    elevenlabs_voice_selector = 'elevenlabs_voice_selector'
    elevenlabs_model_selector = 'elevenlabs_model_selector'
    heygen_voice_selector = 'heygen_voice_selector'
    heygen_avatar_selector = 'heygen_avatar_selector'
    llm_model_selector = 'llm_model_selector'
    blandai_voice_selector = 'blandai_voice_selector'
    d_id_voice_selector = 'd_id_voice_selector'
    d_id_avatar_selector = 'd_id_avatar_selector'
    vapi_custom_phone_number_selector = 'vapi_custom_phone_number_selector'
    vapi_custom_assistant_selector = 'vapi_custom_assistant_selector'
    webflow_collections = 'webflow_collections'


class ValueSuggestionChain(BaseModel):
    url: str
    project_id: str
    output_key: Optional[str] = 'value'


class EnumItem(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    description: str
    value: str
    group_name: Optional[str] = None


class BulkRunInputSource(Enum):
    field_ = ''
    field_DOCUMENT = '$DOCUMENT'
    field_FIELD_PARAM_MAPPING = '$FIELD_PARAM_MAPPING'


class AgentInputSource(Enum):
    conversation_id = 'conversation_id'
    agent_id = 'agent_id'


class Language(Enum):
    python = 'python'
    javascript = 'javascript'
    html = 'html'


class Header(BaseModel):
    hide: Optional[bool] = Field(None, description='Whether to hide all headers.')
    key: Optional[str] = Field(
        None, description='The header displayed above the key column.'
    )
    value: Optional[str] = Field(
        None, description='The header displayed above the value column.'
    )


class Placeholder(BaseModel):
    hide: Optional[bool] = Field(None, description='Whether to hide all placeholders.')
    key: Optional[str] = Field(
        None, description='The placeholder to display in each cell of the key column.'
    )
    value: Optional[str] = Field(
        None, description='The placeholder to display in each cell of the value column.'
    )


class KeyValueInputOpts(BaseModel):
    header: Optional[Header] = Field(
        None, description='Set headers to display above the key and/or value columns.'
    )
    placeholder: Optional[Placeholder] = Field(
        None,
        description='Set placeholder values to display in the key and/or value columns.',
    )
    addButtonText: Optional[str] = Field(
        None,
        description="The text displayed in the 'Add' button that inserts a new pair.",
    )


class Strict(Enum):
    must = 'must'
    should = 'should'
    must_or = 'must_or'


class FilterType(Enum):
    text_match = 'text_match'
    word_match = 'word_match'
    term = 'term'
    terms = 'terms'
    text = 'text'
    texts = 'texts'
    match = 'match'
    contains = 'contains'
    substring = 'substring'
    class_ = 'class'
    category = 'category'
    exact_match = 'exact_match'
    classes = 'classes'
    categories = 'categories'
    exists = 'exists'
    traditional = 'traditional'
    fuzzy = 'fuzzy'
    regexp = 'regexp'
    ids = 'ids'
    date = 'date'
    numeric = 'numeric'
    search = 'search'
    or_ = 'or'
    word_count = 'word_count'
    character_count = 'character_count'
    dedupe_by_value = 'dedupe_by_value'
    match_array = 'match_array'
    random = 'random'
    and_ = 'and'
    size = 'size'


class Filter(BaseModel):
    strict: Optional[Strict] = None
    condition: Optional[str] = None
    case_insensitive: Optional[bool] = None
    field: Optional[str] = None
    filter_type: Optional[FilterType] = None
    condition_value: Optional[Any] = None
    fuzzy: Optional[float] = None
    join: Optional[bool] = None


class Provider(Enum):
    google = 'google'
    microsoft = 'microsoft'
    salesforce = 'salesforce'
    slack = 'slack'
    zendesk = 'zendesk'
    hubspot = 'hubspot'
    linear = 'linear'
    outreach = 'outreach'
    zoom = 'zoom'
    unipile_linkedin = 'unipile_linkedin'
    unipile_whatsapp = 'unipile_whatsapp'
    zoho_crm = 'zoho_crm'


class Type1(Enum):
    email_read_write = 'email-read-write'
    calendar_read_write = 'calendar-read-write'
    microsoft_teams = 'microsoft-teams'
    meeting_read_write = 'meeting-read-write'
    salesforce_api = 'salesforce-api'
    slack_channel_post = 'slack-channel-post'
    slack_channel_post_read = 'slack-channel-post-read'
    zendesk_create_ticket = 'zendesk-create-ticket'
    hubspot_connect_app = 'hubspot-connect-app'
    linear_ticket_create = 'linear-ticket-create'
    unipile_linkedin = 'unipile-linkedin'
    unipile_whatsapp = 'unipile-whatsapp'
    outreach_api = 'outreach-api'
    zoom_api_v1 = 'zoom-api-v1'
    zoho_crm = 'zoho-crm'


class OauthPermission(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    provider: Provider
    types: List[Type1]


class OauthAccountProvider(Enum):
    google = 'google'
    microsoft = 'microsoft'
    salesforce = 'salesforce'
    slack = 'slack'
    zendesk = 'zendesk'
    hubspot = 'hubspot'
    linear = 'linear'
    outreach = 'outreach'
    zoom = 'zoom'
    unipile_linkedin = 'unipile_linkedin'
    unipile_whatsapp = 'unipile_whatsapp'
    zoho_crm = 'zoho_crm'


class OauthAccountPermissionType(Enum):
    email_read_write = 'email-read-write'
    calendar_read_write = 'calendar-read-write'
    microsoft_teams = 'microsoft-teams'
    meeting_read_write = 'meeting-read-write'
    salesforce_api = 'salesforce-api'
    slack_channel_post = 'slack-channel-post'
    slack_channel_post_read = 'slack-channel-post-read'
    zendesk_create_ticket = 'zendesk-create-ticket'
    hubspot_connect_app = 'hubspot-connect-app'
    linear_ticket_create = 'linear-ticket-create'
    unipile_linkedin = 'unipile-linkedin'
    unipile_whatsapp = 'unipile-whatsapp'
    outreach_api = 'outreach-api'
    zoom_api_v1 = 'zoom-api-v1'
    zoho_crm = 'zoho-crm'


class Type2(Enum):
    dynamic = 'dynamic'
    static = 'static'


class Scratchpad(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    type: Type2


class Metadata1(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    content_type: Optional[ContentType] = None
    allow_one_of_variable_mode: Optional[bool] = None
    api_selector_type: Optional[ApiSelectorType] = None
    api_selector_placeholder: Optional[str] = 'Select Option...'
    variable_search_field: Optional[str] = None
    accepted_file_types: Optional[List[str]] = None
    hidden: Optional[bool] = False
    relevance_only: Optional[bool] = False
    advanced: Optional[bool] = False
    placeholder: Optional[Any] = None
    title: Optional[str] = None
    description: Optional[str] = None
    icon_url: Optional[str] = None
    require_toggle: Optional[bool] = None
    dont_substitute: Optional[bool] = False
    min: Optional[float] = None
    max: Optional[float] = None
    value_suggestion_chain: Optional[ValueSuggestionChain] = None
    enum: Optional[List[EnumItem]] = None
    bulk_run_input_source: Optional[BulkRunInputSource] = None
    agent_input_source: Optional[AgentInputSource] = None
    headers: Optional[List[str]] = None
    rows: Optional[float] = None
    can_add_or_remove_columns: Optional[bool] = None
    placeholders: Optional[Dict[str, str]] = None
    language: Optional[Language] = None
    key_value_input_opts: Optional[KeyValueInputOpts] = Field(
        None, description='Props to pass to the KeyValueInput component.'
    )
    knowledge_set_field_name: Optional[str] = Field(
        None,
        description="[KnowledgeEditor] The name of the field in the transformation's param schema containing the knowledge set ID.",
    )
    filters: Optional[List[Filter]] = Field(
        None, description='General filters for the content_type'
    )
    oauth_permissions: Optional[List[OauthPermission]] = Field(
        None,
        description='(Optional) OAuth permissions required for a step. Only applicable for content_type `oauth_token`',
    )
    is_fixed_param: Optional[bool] = None
    is_history_excluded: Optional[bool] = None
    auto_stringify: Optional[bool] = None
    external_name: Optional[str] = Field(
        None,
        description="Field name in external data source (e.g. 'agent_id' in agent conversation metadata)",
    )
    oauth_account_provider: Optional[OauthAccountProvider] = Field(
        None,
        description='Filters the OAuth account selector based on the selected provider',
    )
    oauth_account_permission_type: Optional[OauthAccountPermissionType] = Field(
        None,
        description='Filters the OAuth account selector based on the selected permission type',
    )
    scratchpad: Optional[Scratchpad] = None


class Items(BaseModel):
    type: Optional[str] = None


class Properties1(BaseModel):
    metadata: Optional[Metadata1] = None
    order: Optional[float] = None
    items: Optional[Items] = None


class ParamsSchema(BaseModel):
    metadata: Optional[Metadata] = None
    properties: Optional[Dict[str, Properties1]] = None


class Metadata2(BaseModel):
    field_order: Optional[List[str]] = Field(
        None,
        description='An array of output keys in the order that they should be displayed in the tool builder. Used in the frontend to guarantee tab order.',
    )


class ContentType1(Enum):
    html = 'html'
    chart_js = 'chart.js'
    external_field = 'external_field'


class Metadata3(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    content_type: Optional[ContentType1] = None
    external_name: Optional[str] = Field(
        None,
        description="Field name in external data source (e.g. 'agent_id' in agent conversation metadata)",
    )


class Properties2(BaseModel):
    metadata: Optional[Metadata3] = None


class OutputSchema(BaseModel):
    metadata: Optional[Metadata2] = None
    properties: Optional[Dict[str, Properties2]] = None


class PredictedOutputItem(BaseModel):
    title: Optional[str] = None
    slug: Optional[str] = None
    type: Optional[str] = None


class Frequency(Enum):
    hourly = 'hourly'
    daily = 'daily'
    every_2_minutes = 'every-2-minutes'


class Schedule(BaseModel):
    frequency: Optional[Frequency] = None


class Steps(BaseModel):
    output: Optional[Dict[str, Any]] = None
    executionTime: Optional[float] = None
    results: Optional[List] = Field(
        None,
        description='Only used if the corresponding step has a `foreach` configured',
    )
    skipped: Optional[bool] = Field(
        None, description='Will be true if the step was skipped, and not run'
    )
    skippedItems: Optional[List] = None


class State(BaseModel):
    params: Optional[Dict[str, Any]] = None
    steps: Optional[Dict[str, Steps]] = None


class MaxJobDuration(Enum):
    hours = 'hours'
    minutes = 'minutes'
    synchronous_seconds = 'synchronous_seconds'
    background_seconds = 'background_seconds'


class Metadata4(BaseModel):
    source_studio_id: Optional[str] = None
    source_region: Optional[str] = None
    source_project: Optional[str] = None
    clone_count: Optional[float] = None


class Metrics(BaseModel):
    views: Optional[float] = None
    executions: Optional[float] = None


class ToolType(BaseModel):
    version: Optional[str] = None
    project: Optional[str] = None
    field_id: Optional[str] = Field(None, alias='_id')
    studio_id: str
    public: Optional[bool] = Field(
        False, description='Anyone can view or clone this tool'
    )
    in_marketplace: Optional[bool] = Field(
        False, description='This tool is listed on the tool marketplace'
    )
    insert_date_: Optional[str] = None
    transformations: Optional[Transformations] = None
    template: Optional[Template] = Field(
        None,
        description='Templates support simple expressions on \'if\' and \'params\' fields. Supply an expression in the format: \n\n${{params.my_param + params.my_param2 <= 5 }}\n\nOperators supported: ["!=","==","+","-","&&","||","false","true"]\nEnsure that there is a space between each expression item or it will not evaluate correctly.',
    )
    update_date_: Optional[str] = None
    is_hidden: Optional[bool] = False
    tags: Optional[Tags] = None
    publicly_triggerable: Optional[bool] = Field(
        False, description='Anyone can run this tool'
    )
    machine_user_id: Optional[str] = None
    creator_user_id: Optional[str] = None
    creator_first_name: Optional[str] = None
    creator_last_name: Optional[str] = None
    creator_display_picture: Optional[str] = None
    cover_image: Optional[str] = None
    emoji: Optional[str] = None
    params_schema: Optional[ParamsSchema] = Field(
        None,
        description='A jsonschema superset object that users parameters will be validated against upon execution.',
    )
    output_schema: Optional[OutputSchema] = Field(
        None,
        description='A jsonschema superset object to provide metadata for tool output fields.',
    )
    predicted_output: Optional[List[PredictedOutputItem]] = None
    schedule: Optional[Schedule] = None
    state: Optional[State] = Field(
        None, description='Override the starting state of the studio'
    )
    title: Optional[str] = None
    type: Optional[str] = None
    action_id: Optional[str] = None
    region: Optional[str] = None
    project: Optional[str] = None
    description: Optional[str] = None
    prompt_description: Optional[str] = None
    state_mapping: Optional[Dict[str, str]] = Field(
        None, description='Mapping from alias -> real variable path'
    )
    max_job_duration: Optional[MaxJobDuration] = Field(
        None,
        description='Switching this to hours tells our runner engine to run the job in a way suited for long runs.',
    )
    metadata: Optional[Metadata4] = None
    metrics: Optional[Metrics] = None
    share_id: Optional[str] = None

    def __repr__(self):
        return f"Tool(tool_id=\"{self.studio_id}\", title=\"{self.title}\")"

### Tool Outputs 

class Status(Enum):
    complete = 'complete'
    inprogress = 'inprogress'
    failed = 'failed'
    cancelled = 'cancelled'


class Error(BaseModel):
    body: Optional[str] = None


class CreditsUsedItem(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    credits: float
    name: str
    num_units: Optional[float] = None
    multiplier: Optional[float] = None
    tool_id: Optional[str] = None
    tool_name: Optional[str] = None
    tool_run_id: Optional[str] = None


class ToolOutput(BaseModel):
    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=() 
    )
    output: Dict[str, Any]
    state: Optional[Dict[str, Any]] = None
    status: Status = Field(
        ...,
        description='Status of the workflow. Used for knowing when to send an email notification.',
    )
    errors: List[Error]
    cost: Optional[float] = None
    credits_used: Optional[List[CreditsUsedItem]] = None
    executionTime: float




from __future__ import annotations

from .._client import RelevanceAI, AsyncRelevanceAI
from .._resource import SyncAPIResource, AsyncAPIResource

from ..resources.agent import Agent, AsyncAgent
from typing import List, Optional
from ..types.oauth import *
import webbrowser


class OAuthManager(SyncAPIResource):

    _client: RelevanceAI

    def add_google_integration(self, auto_open: bool = False) -> str:
        path = "auth/oauth/get_url"
        body = {"provider": "google", "types": ["email-read-write"], "redirect_url": ""}
        response = self._post(path, body=body)
        response = OAuth(**response.json())
        if auto_open:
            webbrowser.open(response.auth_url)
        return response.auth_url

    def list_active_integrations(self):
        path = "auth/oauth/accounts/list"
        response = self._post(path)
        # return response.json()
        return ActiveIntegrations(**response.json())




"""
Approve tasks of an agent in bulk
"""

import time
from dotenv import load_dotenv
load_dotenv()

from relevanceai import RelevanceAI

client = RelevanceAI()

agent_id = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"

message_list = [
    "Research the following company: RelevanceAI relevanceai.com",
    "Research the following company: Vividly govividly.com",
    "Research the following company: Airwallex airwallex.com",
    "Research the following company: Lark larksuite.com",
    "Research the following company: SafetyCulture safetyculture.com",
]

task_ids = [] 

my_agent = client.agents.retrieve_agent(agent_id=agent_id)

# triggers the tasks
for message in message_list: 
    task = my_agent.trigger_task(message)
    task_ids.append(task.conversation_id) # save the ids of the triggered tasks
    time.sleep(1)

tool_id = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"

# approving the tasks given a list of task_ids above 
for task_id in task_ids: 
    approved_task = my_agent.approve_task(task_id)

    # approve only a specific tool (optional) -> use the code below instead
    approved_task = my_agent.approve_task(task_id, tool_id=tool_id)










from __future__ import annotations
from typing import Optional, List, Dict, Any, Union
from pydantic import BaseModel, Field

class ParamsBase(BaseModel):
    title: Optional[str] = None
    description: Optional[str] = None
    order: int
    type: str
    value: Optional[Any] = None
    metadata: Optional[Dict[str, Any]] = None
    default: Optional[Any] = None
    required: Optional[bool] = True

class TextParam(ParamsBase):
    type: str = "string"

class LongTextParam(ParamsBase):
    type: str = "string"
    metadata: Dict[str, str] = Field(default_factory=lambda: {"content_type": "long_text"})

class OptionsParam(ParamsBase):
    type: str = "string"
    enum: List[str]
    value: str

class NumberParam(ParamsBase):
    type: str = "number"
    min: Optional[float] = None
    max: Optional[float] = None
    value: float = 0

class CheckboxParam(ParamsBase):
    type: str = "boolean"
    value: bool = False
    default: bool = False

class ListParam(ParamsBase):
    type: str = "array"
    items: Dict[str, str] = Field(default_factory=lambda: {"type": "string"})
    value: List[str] = Field(default_factory=list)

class JsonParam(ParamsBase):
    type: str = "object"
    value: Dict[str, Any] = Field(default_factory=dict)

class JsonListParam(ParamsBase):
    type: str = "array"
    items: Dict[str, str] = Field(default_factory=lambda: {"type": "object"})
    value: List[Dict[str, Any]] = Field(default_factory=list)

class FileParam(ParamsBase):
    type: str = "string"
    value: str = ""
    metadata: Dict[str, Any] = Field(
        default_factory=lambda: {
            "accepted_file_types": []
        }
    )

class FileTextParam(FileParam):
    metadata: Dict[str, Any] = Field(
        default_factory=lambda: {
            "content_type": "file_to_text_llm_friendly",
            "accepted_file_types": []
        }
    )

class FileUrlParam(FileParam):
    metadata: Dict[str, Any] = Field(
        default_factory=lambda: {
            "content_type": "file_url",
            "accepted_file_types": []
        }
    )

class FileUrlsParam(ParamsBase):
    type: str = "array"
    items: Dict[str, str] = Field(default_factory=lambda: {"type": "string"})
    value: List[str] = Field(default_factory=list)
    metadata: Dict[str, Any] = Field(
        default_factory=lambda: {
            "content_type": "file_urls",
            "accepted_file_types": []
        }
    )

class TableParam(ParamsBase):
    type: str = "array"
    items: Dict[str, str] = Field(default_factory=lambda: {"type": "object"})
    value: List[Dict[str, Any]] = Field(default_factory=list)
    metadata: Dict[str, Any] = Field(
        default_factory=lambda: {
            "content_type": "table",
            "headers": []
        }
    )

class ApiKeyParam(ParamsBase):
    type: str = "string"
    value: str = ""
    metadata: Dict[str, Any] = Field(
        default_factory=lambda: {
            "content_type": "api_key"
        }
    )

class KnowledgeTableParam(ParamsBase):
    type: str = "string"
    value: str = ""
    metadata: Dict[str, Any] = Field(
        default_factory=lambda: {
            "content_type": "knowledge_set"
        }
    )

class OAuthAccountParam(ParamsBase):
    type: str = "string"
    metadata: Dict[str, Any] = Field(
        default_factory=lambda: {
            "content_type": "oauth_account"
        }
    )

class ParamsSchema(BaseModel):
    text: TextParam
    long_text: LongTextParam
    options: OptionsParam
    number: NumberParam
    checkbox: CheckboxParam
    list: ListParam
    json_object: JsonParam
    json_list: JsonListParam
    file_text: FileTextParam
    file_url: FileUrlParam
    file_urls: FileUrlsParam
    table: TableParam
    api_key: ApiKeyParam
    knowledge_table: KnowledgeTableParam
    oauth_account_id: OAuthAccountParam


import requests

def get_api_definition():
    base_url = "https://api-f1db6c.stack.tryrelevance.com/latest"
    params = {
        "password": "super secret password"
    }
    response = requests.get(f"{base_url}/api_definition", params=params)
    return response.json()["api_definition"]

def find_endpoint(api_definition, handler_name):
    return next((endpoint for endpoint in api_definition if endpoint.get('handler_name') == handler_name), None)

def find_endpoints_containing(api_definition, name_part):
    return [endpoint for endpoint in api_definition 
            if name_part.lower() in endpoint.get('handler_name', '').lower()]

def get_endpoint_schemas(endpoint):
    if not endpoint:
        return None
    
    schemas = {
        'name': endpoint['handler_name'],
        'path': '/'.join(endpoint['http_payload_routing_pattern']['path_parts']),
        'method': endpoint['http_payload_routing_pattern']['method'],
        'input_schema': endpoint.get('validation_schemas', {}).get('input_body_schema', {}),
        'output_schema': endpoint.get('validation_schemas', {}).get('output_body_schema', {})
    }
    return schemas

if __name__ == "__main__":
    try:
        api_definition = get_api_definition()
        
        # Example: Find specific endpoint and get its schemas
        endpoint = find_endpoint(api_definition, "ListStudios")
        if endpoint:
            schemas = get_endpoint_schemas(endpoint)
            print("\nEndpoint Schemas:")
            print(f"Name: {schemas['name']}")
            print(f"Path: {schemas['path']}")
            print(f"Method: {schemas['method']}")
            print("\nInput Schema:")
            print(schemas['input_schema'])
            print("\nOutput Schema:")
            print(schemas['output_schema'])

    except requests.exceptions.RequestException as e:
        print(f"Error fetching API definition: {e}")



from __future__ import annotations

from .._client import RelevanceAI, AsyncRelevanceAI
from .._resource import SyncAPIResource, AsyncAPIResource
from ..types.tool import ToolType, ToolOutput
from ..resources.tool import Tool, AsyncTool
from typing import List, Optional
import json
import uuid


class ToolsManager(SyncAPIResource):

    _client: RelevanceAI

    def list_tools(
        self,
        max_results: Optional[int] = 100,
    ) -> List[Tool]:
        path = "studios/list"
        params = {
            "filters": json.dumps(
                [
                    {
                        "filter_type": "exact_match",
                        "field": "project",
                        "condition_value": self._client.project,
                        "condition": "==",
                    }
                ]
            ),
            "page_size": max_results,
        }
        response = self._client.get(path, params=params)
        tools = [Tool(client=self._client, **item) for item in response.json().get("results", [])]
        return tools
    
    def retrieve_tool(self, tool_id: str) -> Tool:
        path = f"studios/{tool_id}/get"
        response = self._get(path)
        return Tool(client=self._client, **response.json()["studio"])
    
    def create_tool(
        self, 
        title: str, 
        description: str, 
        public: bool = False,
        params_schema = None,
        output_schema = None,
        transformations = None
    ) -> Tool:
        tool_id = uuid.uuid4()
        path = "studios/bulk_update"
        body = {
            "updates": [
                {
                    "title": title,
                    "public": public,
                    "project": self._client.project,
                    "description": description,
                    "version": "latest",
                    "params_schema": {
                        "properties": {},
                        "required": [],
                        "type": "object"
                    },
                    "output_schema": {},
                    "transformations": {
                        "steps": []
                    },
                    "studio_id": tool_id
                }
            ],
            "partial_update": True
        }
        response = self._post(path, body=body)
        tool_response = self.retrieve_tool(tool_id)
        return tool_response

    def clone_tool(
        self,
        tool_id,
    ) -> Optional[Tool]: 
        path = "/studios/clone"
        body = {
            "studio_id": tool_id,
            "project": self._client.project,
            "region": self._client.region
        }
        response = self._post(path, body=body)
        cloned_tool_id = response.json().get("studio_id", None)
        if cloned_tool_id: 
            tool_response = self.retrieve_tool(cloned_tool_id)
            return tool_response
        else:
            return False

    def delete_tool(self, tool_id: str) -> bool:
        path = "studios/bulk_delete"
        body = {"ids": [tool_id]}
        response = self._post(path, body=body)
        return response.status_code == 200
    

class AsyncToolsManager(AsyncAPIResource):

    _client: AsyncRelevanceAI

    async def list_tools(
        self,
        max_results: Optional[int] = 100,
    ) -> List[AsyncTool]:
        path = "studios/list"
        params = {
            "filters": json.dumps(
                [
                    {
                        "filter_type": "exact_match",
                        "field": "project",
                        "condition_value": self._client.project,
                        "condition": "==",
                    }
                ]
            ),
            "page_size": max_results,
        }
        response = await self._client.get(path, params=params)
        tools = [AsyncTool(client=self._client, **item) for item in response.json().get("results", [])]
        return tools
    
    async def retrieve_tool(self, tool_id: str) -> AsyncTool:
        path = f"studios/{tool_id}/get"
        response = await self._get(path)
        return AsyncTool(client=self._client, **response.json()["studio"])
    
    async def create_tool(
        self, 
        title: str, 
        description: str, 
        public: bool = False,
        params_schema = None,
        output_schema = None,
        transformations = None
    ) -> AsyncTool:
        tool_id = str(uuid.uuid4())  # Convert UUID to string
        path = "studios/bulk_update"
        body = {
            "updates": [
                {
                    "title": title,
                    "public": public,
                    "project": self._client.project,
                    "description": description,
                    "version": "latest",
                    "params_schema": {
                        "properties": {},
                        "required": [],
                        "type": "object"
                    },
                    "output_schema": {},
                    "transformations": {
                        "steps": []
                    },
                    "studio_id": tool_id
                }
            ],
            "partial_update": True
        }
        response = await self._post(path, body=body)
        tool_response = await self.retrieve_tool(tool_id)
        return tool_response
    
    async def clone_tool(
        self,
        tool_id: str,
    ) -> Optional[AsyncTool]:
        path = "/studios/clone"
        body = {
            "studio_id": tool_id,
            "project": self._client.project,
            "region": self._client.region
        }
        response = await self._post(path, body=body)
        cloned_tool_id = response.json().get("studio_id", None)
        if cloned_tool_id:
            tool_response = await self.retrieve_tool(cloned_tool_id)
            return tool_response
        else:
            return False

    async def delete_tool(self, tool_id: str) -> bool:
        path = "studios/bulk_delete"
        body = {"ids": [tool_id]}
        response = await self._post(path, body=body)
        return response.status_code == 200


from setuptools import find_packages, setup

setup(
    name="relevanceai",
    version="10.2.2",
    url="https://relevanceai.com/",
    author="Relevance AI",
    author_email="jacky@relevanceai.com",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    setup_requires=["wheel"],
    install_requires=[
        "pydantic==2.8.2",
        "requests==2.32.3",
        "httpx==0.27.2",
        "pytest==8.3.4"
    ],
    package_data={"": ["*.ini"]},
    extras_require=dict(),
)



from __future__ import annotations
import httpx 
from httpx import Timeout, URL, Headers, Response
from httpx._types import ProxiesTypes

class SyncAPIClient:
    
    _client: httpx.Client

    def __init__(
        self,
        *,
        base_url: str | URL,
        headers: Headers | None = None,
        timeout: float | Timeout | None = None,
        proxies: ProxiesTypes | None = None,
    ) -> None:
        self._client = httpx.Client(base_url=base_url, headers=headers, timeout=timeout, proxies=proxies)

    def request(self, method: str, url: str, **kwargs) -> httpx.Response:
        return self._client.request(method, url, **kwargs)
    
    def get(self, path: str, **kwargs) -> Response:
        return self.request("GET", path, **kwargs)

    def post(self, path: str, body: dict = None, **kwargs) -> Response:
        if body is not None:
            kwargs['json'] = body
        return self.request("POST", path, **kwargs)

    def patch(self, path: str, body: dict = None, **kwargs) -> Response:
        if body is not None:
            kwargs['json'] = body
        return self.request("PATCH", path, **kwargs)

    def put(self, path: str, body: dict = None, **kwargs) -> Response:
        if body is not None:
            kwargs['json'] = body
        return self.request("PUT", path, **kwargs)

    def delete(self, path: str, **kwargs) -> Response:
        return self.request("DELETE", path, **kwargs)

    def close(self) -> None:
        self._client.close()

class AsyncAPIClient:
    _client: httpx.AsyncClient

    def __init__(
        self,
        *,
        base_url: str | URL,
        headers: Headers | None = None,
        timeout: float | Timeout | None = None,
        proxies: ProxiesTypes | None = None,
    ) -> None:
        self._client = httpx.AsyncClient(
            base_url=base_url, 
            headers=headers, 
            timeout=timeout, 
            proxies=proxies
        )

    async def request(self, method: str, url: str, **kwargs) -> httpx.Response:
        return await self._client.request(method, url, **kwargs)
    
    async def get(self, path: str, **kwargs) -> Response:
        return await self.request("GET", path, **kwargs)

    async def post(self, path: str, body: dict = None, **kwargs) -> Response:
        if body is not None:
            kwargs['json'] = body
        return await self.request("POST", path, **kwargs)

    async def patch(self, path: str, body: dict = None, **kwargs) -> Response:
        if body is not None:
            kwargs['json'] = body
        return await self.request("PATCH", path, **kwargs)

    async def put(self, path: str, body: dict = None, **kwargs) -> Response:
        if body is not None:
            kwargs['json'] = body
        return await self.request("PUT", path, **kwargs)

    async def delete(self, path: str, **kwargs) -> Response:
        return await self.request("DELETE", path, **kwargs)

    async def close(self) -> None:
        await self._client.aclose()



"""
Trigger templated tasks. (from .csv file)
"""

import time 
import csv
import time 
from textwrap import dedent

from relevanceai import RelevanceAI
from dotenv import load_dotenv
load_dotenv()

client = RelevanceAI()

agent_id = "xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
my_agent = client.agents.retrieve_agent(agent_id=agent_id)

path = "test_leads.csv"

with open(path, "r") as f:
    reader = csv.reader(f)
    next(reader)  # Skip the header
    for i, r in enumerate(reader):

        print(f"Row {i}...")

        row = {
            "first_name": r[0],
            "last_name": r[1],
            "title": r[2],
            "company": r[3],
            "ls_company_website": r[4],
            "email": r[5],
            "lead_source": r[6],
            "street": r[7],
            "rating": r[8],
            "lead_owner": r[9],
        }
        
        message = dedent(f"""
        Outreach to the following prospect: 
                         
        Full Name: {row["first_name"]} {row["last_name"]}

        Email: {row["email"]}

        Job Title: {row["title"]}

        Company: {row["company"]}

        Company Website: {row["ls_company_website"]}

        Lead Owner: {row["lead_owner"]}
        """)

        task = my_agent.trigger_task(
            message=message,
        )

        print("Triggered task: ", task.conversation_id)

        time.sleep(1)

print("Done!")



from __future__ import annotations

from .._client import RelevanceAI, AsyncRelevanceAI
from .._resource import SyncAPIResource, AsyncAPIResource

from ..resources.agent import Agent, AsyncAgent
from typing import List, Optional


class AgentsManager(SyncAPIResource):

    _client: RelevanceAI

    def list_agents(self) -> List[Agent]:
        path = "agents/list"
        response = self._post(path)
        agents = [
            Agent(client=self._client, **item)
            for item in response.json().get("results", [])
        ]
        return sorted(
            agents, key=lambda x: (x.metadata.name is None, x.metadata.name or "")
        )

    def retrieve_agent(
        self,
        agent_id: str,
    ) -> Agent:
        path = f"agents/{agent_id}/get"
        response = self._get(path)
        return Agent(client=self._client, **response.json()["agent"])

    def create_agent(
        self,
        name: str,
        system_prompt: Optional[str] = None,
        model: Optional[str] = None,
        temperature: Optional[int] = None,
    ) -> Agent:
        existing_agents = self.list_agents()
        for agent in existing_agents:
            if agent.metadata.name == name:
                raise ValueError(f"Agent with name '{name}' already exists!")
        path = f"/agents/upsert"
        body = {
            k: v
            for k, v in {
                "name": name,
                "system_prompt": system_prompt,
                "model": model,
                "temperature": temperature,
            }.items()
            if v is not None
        }
        response = self._post(path, body=body)
        agent = self.retrieve_agent(response.json()["agent_id"])
        return agent

    def upsert_agent(
        self,
        agent_id: Optional[str] = None,
        name: Optional[str] = None,
        system_prompt: Optional[str] = None,
        model: Optional[str] = None,
        temperature: Optional[int] = None,
    ) -> Agent:
        path = f"/agents/upsert"
        body = {
            k: v
            for k, v in {
                "agent_id": agent_id,
                "name": name,
                "system_prompt": system_prompt,
                "model": model,
                "temperature": temperature,
            }.items()
            if v is not None
        }
        response = self._post(path, body=body)
        agent = self.retrieve_agent(response.json()["agent_id"])
        return agent

    def delete_agent(
        self,
        agent_id: str,
    ) -> bool:
        path = f"agents/{agent_id}/delete"
        response = self._post(path)
        return response.status_code == 200


class AsyncAgentsManager(AsyncAPIResource):
    _client: AsyncRelevanceAI

    async def list_agents(self) -> List[AsyncAgent]:
        path = "agents/list"
        response = await self._post(path)
        agents = [
            AsyncAgent(client=self._client, **item)
            for item in (response.json()).get("results", [])
        ]
        return sorted(
            agents, key=lambda x: (x.metadata.name is None, x.metadata.name or "")
        )

    async def retrieve_agent(
        self,
        agent_id: str,
    ) -> AsyncAgent:
        path = f"agents/{agent_id}/get"
        response = await self._get(path)
        return AsyncAgent(client=self._client, **response.json()["agent"])

    async def upsert_agent(
        self,
        agent_id: Optional[str] = None,
        name: Optional[str] = None,
        system_prompt: Optional[str] = None,
        model: Optional[str] = None,
        temperature: Optional[int] = None,
    ) -> AsyncAgent:
        path = f"/agents/upsert"
        body = {
            k: v
            for k, v in {
                "agent_id": agent_id,
                "name": name,
                "system_prompt": system_prompt,
                "model": model,
                "temperature": temperature,
            }.items()
            if v is not None
        }
        response = await self._post(path, body=body)
        agent = await self.retrieve_agent(response.json()["agent_id"])
        return agent

    async def delete_agent(
        self,
        agent_id: str,
    ) -> bool:
        path = f"agents/{agent_id}/delete"
        response = await self._post(path)
        return response.status_code == 200



from __future__ import annotations
import time
from typing import TYPE_CHECKING, Type, TypeVar, Dict, Any, Optional, Union
import httpx
import asyncio
if TYPE_CHECKING:
    from ._client import RelevanceAI, AsyncRelevanceAI

ResponseT = TypeVar('ResponseT')

class SyncAPIResource:
    _client: RelevanceAI
    
    def __init__(self, client: RelevanceAI) -> None:
        self._client = client

    def _get(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        response = self._client.get(path, json=body, params=params)
        return self._cast_response(response, cast_to)

    def _post(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None, 
        options: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        options = options or {}
        response = self._client.post(path, json=body, params=params, **options)
        return self._cast_response(response, cast_to)

    def _patch(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None, 
        options: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        options = options or {}
        response = self._client.patch(path, json=body, params=params, **options)
        return self._cast_response(response, cast_to)

    def _put(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None, 
        options: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        options = options or {}
        response = self._client.put(path, json=body, params=params, **options)
        return self._cast_response(response, cast_to)

    def _delete(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None, 
        options: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        options = options or {}
        response = self._client.delete(path, json=body, params=params, **options)
        return self._cast_response(response, cast_to)

    def _cast_response(
        self, 
        response: httpx.Response, 
        cast_to: Type[ResponseT] = None
    ) -> Union[ResponseT, dict, httpx.Response]:
        if cast_to:
            return response.json() if cast_to == dict else cast_to(**response.json())
        return response

    def _sleep(self, seconds: float) -> None:
        time.sleep(seconds)

class AsyncAPIResource:
    _client: AsyncRelevanceAI
    
    def __init__(self, client: AsyncRelevanceAI) -> None:
        self._client = client

    async def _get(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        response = await self._client.get(path, json=body, params=params)
        return await self._cast_response(response, cast_to)

    async def _post(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None, 
        options: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        options = options or {}
        response = await self._client.post(path, json=body, params=params, **options)
        return await self._cast_response(response, cast_to)

    async def _patch(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None, 
        options: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        options = options or {}
        response = await self._client.patch(path, json=body, params=params, **options)
        return await self._cast_response(response, cast_to)

    async def _put(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None, 
        options: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        options = options or {}
        response = await self._client.put(path, json=body, params=params, **options)
        return await self._cast_response(response, cast_to)

    async def _delete(
        self, 
        path: str, 
        cast_to: Type[ResponseT] = None, 
        body: Optional[Dict[str, Any]] = None, 
        params: Optional[Dict[str, Any]] = None, 
        options: Optional[Dict[str, Any]] = None
    ) -> ResponseT:
        options = options or {}
        response = await self._client.delete(path, json=body, params=params, **options)
        return await self._cast_response(response, cast_to)

    async def _cast_response(
        self, 
        response: httpx.Response, 
        cast_to: Type[ResponseT] = None
    ) -> Union[ResponseT, dict, httpx.Response]:
        if cast_to:
            json_data = await response.json()
            return json_data if cast_to == dict else cast_to(**json_data)
        return response

    async def _sleep(self, seconds: float) -> None:
        await asyncio.sleep(seconds)


from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Literal, Optional, Union

from pydantic import BaseModel, ConfigDict, Field


class OAuth(BaseModel):
    auth_url: str
    scopes: Optional[List[str]] = None


from pydantic import BaseModel, Field
from typing import List, Dict, Optional
from datetime import datetime


class Token(BaseModel):
    permission_types: List[str]
    scopes: List[str]
    token_id: str
    metadata: Dict[str, str]
    insert_date_: datetime


class Account(BaseModel):
    _id: str
    project: str
    provider: str
    account_id: str
    provider_user_id: str
    insert_date_: datetime
    update_date_: datetime
    tokens: List[Token]
    metadata: Dict[str, str] = Field(default_factory=dict)
    label: str

    def __repr__(self) -> str:
        return f"Integration(provider='{self.provider}, email='{self.label}'"


class ActiveIntegrations(BaseModel):
    results: List[Account]

    def __getattr__(self, provider: str) -> Optional[Account]:
        if provider in ["google"]:  # Add supported providers
            return next((r for r in self.results if r.provider == provider), None)
        raise AttributeError(
            f"'{self.__class__.__name__}' has no attribute '{provider}'"
        )



import pytest
from unittest.mock import MagicMock
from relevanceai.resources.tasks import Tasks
from relevanceai.types.knowledge import Metadata
from relevanceai.types.task import TaskMetadata

class TestTasks:
    @pytest.fixture
    def mock_client(self):
        """Fixture to create a mock RelevanceAI client."""
        return MagicMock()

    @pytest.fixture
    def tasks(self, mock_client):
        """Fixture to create a Tasks instance."""
        return Tasks(client=mock_client)

    def test_get_metadata_without_metadata(self, tasks):
        """Test getting metadata when no metadata exists in response."""
        mock_response = MagicMock()
        mock_response.json.return_value = {"metadata": None}
        tasks._get = MagicMock(return_value=mock_response)

        result = tasks.get_metadata("conversation-123")

        tasks._get.assert_called_once_with(
            "knowledge/sets/conversation-123/get_metadata"
        )
        assert isinstance(result, dict)
        assert result == {}

    def test_get_metadata_empty_response(self, tasks):
        """Test getting metadata with empty response."""
        mock_response = MagicMock()
        mock_response.json.return_value = {}
        tasks._get = MagicMock(return_value=mock_response)

        result = tasks.get_metadata("conversation-123")

        tasks._get.assert_called_once_with(
            "knowledge/sets/conversation-123/get_metadata"
        )
        assert isinstance(result, dict)
        assert result == {}

    def test_delete_task_success(self, tasks):
        """Test successful task deletion."""
        mock_response = MagicMock()
        mock_response.status_code = 200
        tasks._post = MagicMock(return_value=mock_response)

        result = tasks.delete_task("conversation-123")

        tasks._post.assert_called_once_with(
            path="knowledge/sets/delete",
            body={"knowledge_set": ["conversation-123"]}
        )
        assert result is True

    def test_delete_task_failure(self, tasks):
        """Test failed task deletion."""
        mock_response = MagicMock()
        mock_response.status_code = 404
        tasks._post = MagicMock(return_value=mock_response)

        result = tasks.delete_task("conversation-123")

        tasks._post.assert_called_once_with(
            path="knowledge/sets/delete",
            body={"knowledge_set": ["conversation-123"]}
        )
        assert result is False


from __future__ import annotations
from typing import Optional, Dict, Any, List
from pydantic import BaseModel, Field, ConfigDict


class TransformationBase(BaseModel):
    transformation: str
    name: str
    params: Dict[str, Any]
    output: Optional[Dict[str, str]] = None
    display_name: Optional[str] = None

class PromptCompletionTransformation(TransformationBase):
    transformation: str = "prompt_completion"
    name: str = "llm"
    params: Dict[str, Any] = Field(
        ..., 
        example={
            "prompt": "Tell me about AI Agents",
            "model": "openai-gpt4o"
        }
    )

class PythonCodeTransformation(TransformationBase):
    transformation: str = "python_code_transformation"
    name: str = "python"
    params: Dict[str, Any] = Field(
        ...,
        example={
            "code": "\nreturn \"Hello World!\""
        }
    )

class SerperGoogleSearchTransformation(TransformationBase):
    transformation: str = "serper_google_search"
    name: str = "google"
    params: Dict[str, Any] = Field(
        ...,
        example={
            "search_query": "Relevance AI"
        }
    )

class TransformationStep(BaseModel):
    steps: List[TransformationBase]

    model_config = ConfigDict(
        extra='allow',
        protected_namespaces=()
    )